doc="""Supporting tool for publication Joule *Heating in Nanowires*, by 

Hans Fangohr, Dmitri S. Chernyshenko, Matteo Franchin, Thomas
Fischbacher, Guido Meier, Joule heating in nanowires,
Physical Review B, 85, 054437 (2011)
Online: 
- Published version: http://prb.aps.org/abstract/PRB/v84/i5/e054437
- Preprint: http://arxiv.org/abs/1012.4304
- Preprint: http://eprints.soton.ac.uk/184837/

Please see
http://www.soton.ac.uk/~fangohr/publications/2011-joule-heating-in-nanowires.html
for an up-to-date reference and newer versions of this tool. 

All equations below refer to this publication.

  Disclaimer: The authors nor anybody else assume any responsibility
  whatsoever for the use of this too by other parties, and makes no
  guarantees, expressed or implied, about its quality, reliability, or
  any other characteristic.

  $Id: 3d.py,v 7950bc7b47f0 2012/05/30 23:25:05 fangohr $

Notes:
  * this program should only be used for a three-dimensional substrate
    (i.e. thickness of substrate >> length of wire)

  * the approximation used is only valid up to a time of the order of tc
"""

import sys              #Standard Python Library

#import objects we need from nanowirelib
try:
    import nanowirelib
except ImportError:     
    print "Can't find file 'nanowirelib.py'. Needs to be in the same directory."
    print "All files can be downloaded from"
    print "http://www.soton.ac.uk/~fangohr/publications/2011-joule-heating-in-nanowires.html"
    print "Stopping here; sorry."
    sys.exit(1)

import time                                                  #Python's time module
from numpy import logspace,log10                             #from numpy Python module
from nanowirelib import T3d, estimate_T3d_max_t, zigzagdata  #from file 'nanowirelib.py'

#Should we show a plot of the graph and display on screen? ('True' for yes, 'False' for no)
show_plot = True

#Should we save a pdf and png image of the graph? (Can also do interactively in figure window)
#Will be saved to file 'T_3d_substrate.pdf' and 'T_3d_substrate.png'
save_plot = True
filenameroot = 'T_3d_substrate'

#Should we save the data in a table in a text file (saved to 'T_3d_substrate.txt').
save_data_to_textfile = True

#Should we show the finite element simulation for comparison (zigzag wire, Sect 3E)
show_finite_element_simulation_data = True

#Generic constants
nm=1e-9   #define 1 nanometre to be 10^{-9}

#define problem constants (enter your own values here)

#Experimental parameters (as in section 3E "Zigzag-shapen nanowire"),
#but you can modify these to match your own experiment.

j=2.2e12        #current density (A/m^2)
w=500*nm        #width of nanowire (m)
h=10*nm         #height (=thickness) of nanowire (m)
k=148           #thermal conductivity substrate (W/(m K))
sigma= 1./42e-8#electric conductivity nanowire (1/(Ohm m) = A^2 s^3 / kg m^3)
rho= 2333.      #density of substrate (kg/m^3)
C=714.          #specific heat capacity of substrate (J/(kg K))
L=56e-6         #length of wire (m)
                #For zigzag wire use effective length as a straight line
                #(projection of wire on x-axis)

#Parameters for plot/data output can be changed below (look for 'min_t' and 'max_t').

print "==== (Program starts at %s) ====" % time.asctime()
print #prints empty line
print doc
print 
print "Parameters used in this run (all in SI units, see source for dimensions) are:"+\
      "\n\tj=%g\n\tw=%g\n\th=%g\n\tk=%g\n\tsigma=%g\n\trho=%g\n\tC=%g\n\tL=%g" % \
      (j,w,h,k,sigma,rho,C,L)
print
print "For these parameters, we compute the maximum substrate temperature increase"
print "as a function of time (using Eq 4) and save to file/plot as requested."
print "We also compute the time tc up to which this expression is expected to"
print "be valid (Eq 5):"
print 

#estimate validity of model (using Equation 5)
tc = estimate_T3d_max_t(rho,k,C,L)
#compute max substrate temperature T_Tc=T(tc) at that time tc
T_tc = T3d(tc,j,w,h,k,sigma,rho,C,alpha=0.5)
print "Estimate tc=%.4g seconds (Eq 6)" % tc
print "At t=tc, we find Delta T(tc)=%.4g Kelvin (Eq 5)" % T_tc
print

#create array of values of t starting from min_t up to max_t using 200 steps and log spacing
min_t = 1e-12  #seconds
max_t = tc*1.1 #go ten percent above estimated limit of applicability.
               #this value can be changed explicitely if desired.
t = logspace(log10(min_t),log10(max_t),200)

#compute the corresponding temperatures using the analytical equation for
#3d substrate (You etal 2006)
T3ddata = T3d(t,j,w,h,k,sigma,rho,C,alpha=0.5)

assert len(t) == len(T3ddata), "Internal error, please report to authors"

if save_data_to_textfile:
    filename = filenameroot+'.txt'
    f=open(filename,'w')
    for i in xrange(len(t)):
        f.write('%15g\t%15g\n' % (t[i],T3ddata[i]))
    f.close()
    print "-> Have saved data to %s (tab-separated)" % filename

if show_plot or save_plot:
    import matplotlib.pyplot as plt
    plt.semilogx(t,T3ddata,'-',label='analytical approx (eq 4) for 3d substrate',lw=3)
    plt.xlabel('time [s]')
    plt.ylabel('Delta T [K]')
    plt.title('Plot produced with 3d.py on %s' % time.asctime())
    plt.grid()

    #fake data set to show where tc is on plot
    x_tc = [tc,tc]
    y_tc = [0.5*T_tc,1.5*T_tc]
    plt.semilogx(x_tc,y_tc,'-^m',label='tc=%.1g s'%tc)

    #if desired, plot FE solution. Makes only sense for parameters
    #for zigzag nanowire simulation as in Sec 3E of publication.)
    if show_finite_element_simulation_data:
        t,maxTwire,maxTsubstrate,minTsubstrate = zigzagdata()
        plt.semilogx(t,maxTwire,'-',lw=1.5,label='max(T) in wire (FE result Sec 3E)')
        plt.semilogx(t,maxTsubstrate,'--',label='max(T) in substrate (FE result Sec 3E)')
        plt.semilogx(t,minTsubstrate,'-',label='min(T) in substrate (FE result Sec 3E)')

    plt.legend(loc='upper left')
    if save_plot:
        plt.savefig(filenameroot+'.png')
        print "-> Have created plot and saved to %s.png." % (filenameroot)
        plt.savefig(filenameroot+'.pdf')
        print "-> Have created plot and saved to %s.pdf." % (filenameroot)
    if show_plot:
        print "Press CTRL+C to stop program, or close figure window"
        plt.show()

print
print "==== Program completed ===="
print "If you use results obtained with this tool in publications,"
print "please cite Physical Review B, 85, 054437 (2011)
print "==========================="
 
